#!/usr/share/ucs-test/runner pytest-3 -s -l -v
## -*- coding: utf-8 -*-
## desc: Check id broker settings and cmdline tool
## tags: [ucsschool-id-broker]
## packages: [id-broker-common]
## exposure: dangerous

import json
import subprocess

import pytest
from utils import _env_is_not_empty

import univention.testing.strings as uts
import univention.testing.ucsschool.ucs_test_school as utu
from univention.config_registry import ConfigRegistry
from univention.idbroker.lib import generate_pseudonym
from univention.idbroker.udm_host import get_settings_data_content
from univention.udm.encoders import Base64Bzip2BinaryProperty

pytestmark = pytest.mark.skipif(
    _env_is_not_empty(),
    reason="This test assumes and environment without provisioned users, school_classes and ous.",
)

@pytest.fixture()
def ldap_base():
    return ConfigRegistry().load().get("ldap/base")


@pytest.fixture()
def mappings_settings_data_dn(ldap_base):
    return f"cn=IDBrokerServiceProviderMappings,cn=IDBrokerSettings,cn=univention,{ldap_base}"


@pytest.fixture()
def secrets_settings_data_dn(ldap_base):
    return f"cn=IDBrokerServiceProviderSecrets,cn=IDBrokerSettings,cn=univention,{ldap_base}"


@pytest.fixture()
def preserve_settings_data(udm):
    settings = []

    def _preserve_settings_data(dn):
        obj = udm.obj_by_dn(dn)
        settings.append((dn, obj.props.data.raw))
        obj.props.data = Base64Bzip2BinaryProperty("data", raw_value=b"{}")
        obj.save()

    yield _preserve_settings_data
    for dn, raw_data in settings:
        obj = udm.obj_by_dn(dn)
        obj.props.data = Base64Bzip2BinaryProperty("data", raw_value=raw_data)
        obj.save()

@pytest.mark.parametrize(
    "setting,settings_dn",
    [
        ("secrets", "secrets_settings_data_dn"),
        ("mappings", "mappings_settings_data_dn"),
    ],
)
def test_add_remove_setting(udm, setting, settings_dn, preserve_settings_data, request):
    settings_dn = request.getfixturevalue(settings_dn)
    preserve_settings_data(settings_dn)

    key = uts.random_string()
    value = "idBrokerPseudonym0001"
    expected_data = {}
    expected_data[key] = value

    subprocess.run(
        [
            "manage-service-providers",
            "addsetting",
            setting,
            "--key",
            key,
            "--value",
            value,
        ]
    )
    res = subprocess.run(
        ["manage-service-providers", "showsetting", setting],
        capture_output=True,
    )
    res = json.loads(res.stdout)
    assert expected_data == res

    ldap_data = udm.obj_by_dn(settings_dn)
    ldap_data = json.loads(ldap_data.props.data.raw)

    assert expected_data == ldap_data
    assert ldap_data == res

    expected_data = {}
    subprocess.run(
        [
            "manage-service-providers",
            "removesetting",
            setting,
            "--key",
            key,
        ]
    )
    res = subprocess.run(
        ["manage-service-providers", "showsetting", setting],
        capture_output=True,
    )
    res = json.loads(res.stdout)
    assert expected_data == res

    ldap_data = udm.obj_by_dn(settings_dn)
    ldap_data = json.loads(ldap_data.props.data.raw)

    assert expected_data == ldap_data
    assert ldap_data == res


def test_add_remove_service_provider(
    udm, mappings_settings_data_dn, secrets_settings_data_dn, preserve_settings_data
):
    preserve_settings_data(mappings_settings_data_dn)
    preserve_settings_data(secrets_settings_data_dn)
    school_auth = uts.random_string()
    with utu.UCSTestSchool() as schoolenv:
        ou_name, ou_dn = schoolenv.create_ou(name_edudc=schoolenv.ucr["hostname"])
        student_name, student_dn = schoolenv.create_user(ou_name)
        workgroup_name, workgroup_dn = schoolenv.create_workgroup(ou_name)
        schoolclass_name, schoolclass_dn = schoolenv.create_school_class(ou_name)

        dn_list = [ou_dn, student_dn, workgroup_dn, schoolclass_dn]
        for obj_dn in dn_list:
            if not obj_dn == workgroup_dn:
                obj = udm.obj_by_dn(obj_dn)
                obj.props.ucsschoolSourceUID = school_auth
                obj.props.ucsschoolRecordUID = uts.random_string()
                obj.save()
            assert obj.props.idBrokerPseudonym0001 is None

        mappings = get_settings_data_content(mappings_settings_data_dn)
        assert mappings == {}
        secrets = get_settings_data_content(secrets_settings_data_dn)
        assert secrets == {}

        service_provider = uts.random_string()
        secret = uts.random_string()
        subprocess.run(
            [
                "manage-service-providers",
                "addservice",
                "--name",
                service_provider,
                "--secret",
                secret,
            ]
        )
        for obj_dn in dn_list:
            provisioned_obj = udm.obj_by_dn(obj_dn)
            # not for workgroups
            if not obj_dn == workgroup_dn:
                obj_record_uid = provisioned_obj.props.ucsschoolRecordUID
                obj_authority = provisioned_obj.props.ucsschoolSourceUID
                pseudonym = generate_pseudonym(secret, obj_record_uid, obj_authority)
                assert provisioned_obj.props.idBrokerPseudonym0001 == pseudonym
            else:
                assert provisioned_obj.props.idBrokerPseudonym0001 is None

        mappings = get_settings_data_content(mappings_settings_data_dn)
        assert mappings == {service_provider: "idBrokerPseudonym0001"}
        secrets = get_settings_data_content(secrets_settings_data_dn)
        assert secrets == {service_provider: secret}

        subprocess.run(
            [
                "manage-service-providers",
                "removeservice",
                "--name",
                service_provider,
            ]
        )

        for obj_dn in dn_list:
            provisioned_obj = udm.obj_by_dn(obj_dn)
            assert provisioned_obj.props.idBrokerPseudonym0001 is None

        mappings = get_settings_data_content(mappings_settings_data_dn)
        assert mappings == {}
        secrets = get_settings_data_content(secrets_settings_data_dn)
        assert secrets == {}
